"""Generates a player profile using markov chains!
"""
import random

import sujmarkov

from random_cricket_profiles import countries, players


class PlayerGenerator():
    def __init__(self, sample_players, min_profile_length):
        self.min_profile_length = min_profile_length
        self.profile_markov = sujmarkov.Markov(n=3)

        self.surname_markovs = {}
        self.firstname_markovs = {}
        for country in countries.COUNTRIES:
            self.surname_markovs[country.country_id] = sujmarkov.Markov(n=4)
            self.firstname_markovs[country.country_id] = sujmarkov.Markov(n=4)

        for player in sample_players:
            self.add_player(player)

    def add_player(self, player):
        for line in player.profile:
            if line:
                sentence = line.split(" ")
                self.profile_markov.add(sentence)

        country_id = player.country_id

        self.surname_markovs[country_id].add(player.surname)

        firstnames = player.firstnames
        if firstnames:
            for name in firstnames.split(" "):
                if name:
                    self.firstname_markovs[country_id].add(name)

    def generate(self, country_code=None, seed=None):
        """Returns a tuple (player, seed).
        If country_code is not passed, a random one is chosed.
        seed is used to seed the random number generator.
        This means that the same seed will always generate the same player.
        """
        if seed is None:
            seed = random.getrandbits(64)

        random_ = random.Random(seed)

        if country_code:
            country = countries.get_country_by_code(country_code)
        else:
            country = random_.choice(countries.COUNTRIES)

        surname_markov = self.surname_markovs[country.country_id]
        surname = "".join(surname_markov.generate(random_=random_))

        firstname_markov = self.firstname_markovs[country.country_id]
        firstnames_as_list = []
        for i in range(random_.choice([1, 2, 3])):
            firstname = "".join(firstname_markov.generate(random_=random_))
            firstnames_as_list.append(firstname)
        firstnames = " ".join(firstnames_as_list)

        profile = []
        while get_total_length(profile) < self.min_profile_length:
            line = " ".join(self.profile_markov.generate(random_=random_))

            for item in [
                ("$fullname", surname),
                ("$known_as", surname),
                ("$surname", surname),
                ("$firstnames", firstnames),
                ("$team", country.name),
            ]:
                placeholder, value = item
                line = line.replace(placeholder, value)
            profile.append(line)

        player = players.Player(
            country_id=country.country_id,
            firstnames=firstnames,
            surname=surname,
            profile=profile,
            fullname=firstnames + " " + surname,
            known_as=""
        )
        return (player, seed)


def get_total_length(profile):
    """Return the sum of lengths of each string in this list.

    >>> get_total_length(["This is a conversation", "Yes"])
    25
    """
    lengths = [len(line) for line in profile]
    return sum(lengths)
