"""Handles loading and processing of player data.
"""
import collections
import gzip
import re

import msgpack

from random_cricket_profiles import countries


Player = collections.namedtuple("Player", ["country_id", "firstnames", "surname", "profile", "fullname", "known_as"])


def load_players(input_filename):
    with gzip.open(input_filename, "rb") as input_file:
        for player_dict in msgpack.unpack(input_file, encoding="utf-8"):
            if "firstnames" not in player_dict:
                player_dict["firstnames"] = ""
            player = Player(**player_dict)
            yield process_player(player)


def process_player(player):
    """Take a player and
    - replace all player details in the profile with placeholders
    - split profiles into sentences (using full-stops)
    """
    country = countries.get_country_by_id(player.country_id)

    # We collapse the profile into one string to make processing simpler.
    one_line_profile = " ".join(player.profile)
    one_line_profile = add_placeholders(one_line_profile, player, country)

    # We will later split into 'sentences' using full-stops. However there
    # are a few pesky initialed names (e.g. M. J. K. Smith) which might
    # confuse the splitting. Assuming a single letter followed by a full-stop
    # seems reasonable.
    one_line_profile = re.sub(r"\b([A-Z])\.", "\1", one_line_profile)

    new_profile = [line.strip() for line in one_line_profile.split(".") if is_profile_line_ok(line)]

    return Player(
        country_id=player.country_id,
        firstnames=get_firstnames(player.fullname, player.surname),
        surname=player.surname,
        profile=new_profile,
        fullname=player.fullname,
        known_as=player.known_as
    )


def add_placeholders(profile_line, player, country):
    """Given some a line, replace all specific details with placeholders.

    i.e. turns
    'Mark Alan Butcher played 71 test matches for England'
    into
    '$firstnames $surname played 71 test matches for $team'
    """
    for (placeholder, value) in [("$fullname", player.fullname),
                                 ("$known_as", player.known_as),
                                 ("$surname", player.surname),
                                 ("$firstnames", player.firstnames),
                                 ("$team", country.name)]:
        if not value:
            continue

        profile_line = profile_line.replace(value, placeholder)

    return profile_line


def get_firstnames(fullname, surname):
    """Assume that the fullname ends with the surname, so the rest must be the
    firstnames.

    >>> get_firstnames("Jean Paul de Beaumarche", "de Beaumarche")
    'Jean Paul'
    >>> get_firstnames("Something Odd", "Smith")
    'Something Odd'
    """
    if not fullname.endswith(surname):
        return fullname
    end = len(fullname) - len(surname)
    firstnames = fullname[:end]
    return firstnames.strip()


def is_profile_line_ok(profile_line):
    """Return true if the profile line is 'ok'.
    We use this to remove lines that we would never want in a profile.
    """
    for banned_word in ["die", "died", "cancer", "death"]:
        pattern = re.compile(r"\b{0}\b".format(banned_word))
        if pattern.search(profile_line.strip().lower()):
            return False
    return bool(profile_line)
